/**
 * ==================================================================
 *
 * Program Name
 *  PutQuit - Put a 'QUIT' Message to a target queue.
 *
 * Last date of modification
 *  February 14, 2011
 *
 * Description
 *  This program will write (put) a 'QUIT' Message to a target queue.
 *  The message's Feedback attribute will be set to MQFB_QUIT.
 *  The program will set the format field to MQ_STR
 *
 *  The following is the logic in this program:
 *
 *  - Initialize and allocated memory.
 *  - Get the command line parameters.
 *  - Connect to the queue manager.
 *  - Open output queue
 *  - put the message
 *  - Close output queue
 *  - Disconnect from the queue manager.
 *  - Terminate
 *
 * @copyright Copyright(C), 1996-2011, Roger Lacroix
 * @author Roger Lacroix, Capitalware Inc.
 * @return 0 for ok or -1 for error
 * @version 1.0.0
 * @license Apache 2 License
 *
 * ------------------------------------------------------------------
 */
#if defined(__MVS__)
#define MVS /* define our MVS indicator */
#define _XOPEN_SOURCE_EXTENDED 1
#endif

/**
 * Program build time eye-catchers for executable.
 */
static const char * EYECATCHER_EyeCatcher  = "PutQuit " ;
static const char * EYECATCHER_Version     = "01.00.00" ;
static const char * EYECATCHER_Datestamp   = __DATE__ ;
static const char * EYECATCHER_Timestamp   = __TIME__ ;
static const char * EYECATCHER_Filename    = __FILE__ ;

/**
 * Include the C standard header files
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
   /* includes for MQI */
#include <cmqc.h>

char* usage = " -m qmgr -q queue   \n"\
"-m qmgr           The name of the local queue manager      \n"\
"-q queue          The name of the output queue             \n"\
"-?                 Display this help message            [Optional]\n";

/**
 * Function Protypes
 */
void GetParameters(int  argc, char *argv[]);

/**
 * Global variables
 */
MQCHAR   QMName[MQ_Q_MGR_NAME_LENGTH];  // queue manager name
MQCHAR   QName[MQ_Q_NAME_LENGTH];       // queue manager name

/**
 * Main line
 */
int main(int argc, char **argv)
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   MQOD     od = {MQOD_DEFAULT};    /* Object Descriptor             */
   MQMD     md = {MQMD_DEFAULT};    /* Message Descriptor            */
   MQPMO   pmo = {MQPMO_DEFAULT};   /* put message options           */

   MQHCONN  Hcon;                   /* connection handle             */
   MQHOBJ   Hobj;                   /* object handle                 */
   MQLONG   O_options;              /* MQOPEN options                */
   MQLONG   C_options;              /* MQCLOSE options               */
   MQLONG   CompCode;               /* completion code               */
   MQLONG   OpenCode;               /* MQOPEN completion code        */
   MQLONG   Reason;                 /* reason code                   */
   MQLONG   CReason;                /* reason code for MQCONN        */
   MQLONG   buflen;                 /* buffer length                 */
   char     QuitMsgBuf[100] = "QUIT";

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   /* printf( "argc=%d\n", argc ); */

   if ( argc != 5)
   {
      printf( "Usage:\n%s%s\n", argv[0], usage );
      exit(-1);
   }

   printf("Starting PutQuit\n");

   /******************************************************************/
   /*   Clear variables and retrieve command-line parameters.        */
   /******************************************************************/

   memset( QMName, ' ', MQ_Q_NAME_LENGTH );
   memset( QName,  ' ', MQ_Q_NAME_LENGTH );

   GetParameters(argc, argv);

   /******************************************************************/
   /*   Connect to queue manager                                     */
   /******************************************************************/

   MQCONN(QMName,                  /* queue manager                  */
          &Hcon,                   /* connection handle              */
          &CompCode,               /* completion code                */
          &CReason);               /* reason code                    */

   if (CompCode == MQCC_FAILED)
   {
      /* report reason and stop if it failed     */
      printf("MQCONN ended with cc %d rc %d\n", CompCode, CReason);
      exit( (int)CReason );
   }

   /******************************************************************/
   /*   -q parameter is for target queue name                        */
   /******************************************************************/
   strncpy(od.ObjectName, QName, (size_t)MQ_Q_NAME_LENGTH);

/*   printf("Target queue is %s\n", od.ObjectName); */

   /******************************************************************/
   /*   Open the target message queue for output                     */
   /******************************************************************/
   O_options = MQOO_OUTPUT           /* open queue for output        */
               + MQOO_FAIL_IF_QUIESCING;

   MQOPEN(Hcon,                      /* connection handle            */
          &od,                       /* object descriptor for queue  */
          O_options,                 /* open options                 */
          &Hobj,                     /* object handle                */
          &OpenCode,                 /* MQOPEN completion code       */
          &Reason);                  /* reason code                  */

   if (OpenCode == MQCC_FAILED)
   {
      /* report reason, if any; stop if failed      */
      printf("MQOPEN ended with reason code %d\n", Reason);
      exit( (int)Reason );
   }

   /****************************************************************/
   /*   Put the 'QUIT' message to the target queue.                */
   /****************************************************************/

   buflen = strlen(QuitMsgBuf);
   md.Feedback = MQFB_QUIT;
   md.MsgType  = MQMT_DATAGRAM;

   memset(md.ReplyToQ, ' ', MQ_Q_NAME_LENGTH);

   memcpy(md.CorrelId, MQCI_NONE, sizeof(md.CorrelId));
   memcpy(md.MsgId, MQMI_NONE, sizeof(md.MsgId));

   memcpy(md.Format,
          MQFMT_STRING, (size_t)MQ_FORMAT_LENGTH);

   MQPUT(Hcon,                /* connection handle               */
         Hobj,                /* object handle                   */
         &md,                 /* message descriptor              */
         &pmo,                /* default options (datagram)      */
         buflen,              /* buffer length                   */
         QuitMsgBuf,          /* message buffer                  */
         &CompCode,           /* completion code                 */
         &Reason);            /* reason code                     */

    /* report reason, if any */
    printf("MQPUT ended with reason code %d\n", Reason);

   /******************************************************************/
   /*   Close the target queue (if it was opened)                    */
   /******************************************************************/
   if (OpenCode != MQCC_FAILED)
   {
      C_options = 0;                  /* no close options           */
      MQCLOSE(Hcon,                   /* connection handle          */
              &Hobj,                 /* object handle               */
              C_options,
              &CompCode,             /* completion code             */
              &Reason);              /* reason code                 */

      if (CompCode != MQCC_OK)
      {
         printf("MQCLOSE ended with reason code %d\n", Reason);
      }
   }

   /******************************************************************/
   /*   Disconnect from Quue Manager if not already connected        */
   /******************************************************************/
   if (CReason != MQRC_ALREADY_CONNECTED)
   {
      MQDISC(&Hcon,                   /* connection handle           */
             &CompCode,               /* completion code             */
             &Reason);                /* reason code                 */

      if (CompCode != MQCC_OK)
      {
         printf("MQDISC ended with reason code %d\n", Reason);
      }
   }

   printf("PutQuit is ending.\n");
   return(0);
 }

/**
 * ==================================================================
 *
 * Function Name
 *  GetCmdLineParms
 *
 * Description
 *  This function will parse the inputted command line and
 *  extract the keyswords and their values.
 *
 * Input parameters
 *  argc - The number of parameters
 *  argv - An array of keywords and values.
 *
 * Output
 *  QMName  - Queue Manager Name
 *  QName   - Target Queue Name
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
void GetParameters(int argc, char *argv[])
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
    #define MaxParams 4

    char *ParmkeyUC[MaxParams] = { "-M", "-Q", ",","-?"};
    char *ParmkeyLC[MaxParams] = { "-m", "-q", ",","-?"};
    int i,j;
    int rc=0;

    /* --------------------------------------------
     * Code section
     * --------------------------------------------
     */

     /* Prior to processing the parameters, establish the default */
     /* values this is done before calling GetParameters          */

    if (argc >1)
    {
        for (i=1; i<argc; i++)
        {
            for (j=0; j<MaxParams; j++)
            {
                if ( (!strncmp( argv[i], ParmkeyUC[j], 2)) ||
                     (!strncmp( argv[i], ParmkeyLC[j], 2)) )
                {
                    break; /* fall out to switch stmt */
                }
            };

            switch (j)
            {
                case 0:                      /* -m */
                    if (i+1 < argc)
                    {
                        strncpy(QMName, argv[i+1], MQ_Q_NAME_LENGTH);
                        i++;
                    }
                    else
                    {
                        rc=9;
                    }
                    break;

                case 1:                      /* -q */
                    if (i+1 < argc)
                    {
                        strncpy(QName, argv[i+1], MQ_Q_NAME_LENGTH);
                        i++;
                    }
                    else
                    {
                        rc=9;
                    }
                    break;

               case 2:                      /* , */
                    /* ignore the comma */
                    break;

               case 3:                      /* -? */
                    exit(-1);
                    break;

               default:/* parameter error */
                    printf( "Error with keyword parameter!\n" );
                    printf( "Parameter=%s\n", argv[i] );
                    exit(-1);
                    break;
                    /* ignore it */
            };
        };
    }

    if ( *(QName) == ' ' || *(QMName) == ' ' )
    {
       printf( "Usage:\n%s%s\n", argv[0], usage );
       exit(-1);
    }

    if ( rc==9 )
    {
       printf( "Usage:\n%s%s\n", argv[0], usage );
       exit(-1);
    }

return;                       /* No parameters given            */
}

/* End of file */
